% Kite parameters used for the simulink model KiteModel.slx
clear; close;
% Simulation parameters

rsb = 0; %1e-10;    % Singularity bias for 1/tan in dhteta
T_sim = 10;%2.3;%4.34;         % Simulation time [s]
plotter = 1;    % plot trajectory or not?
plotTetherTension = 1;  % Plot the tether tension?
figflag = 1;
savefigs = 1;

%E = 3;  % L/D-ratio
CL = 1.5; % Lift coefficient
E = 3;
CD = CL/E;  % Drag coefficient
beta = atan2(CD,CL);    % L-D coefficient
cbeta = cos(beta);
sbeta = sin(beta);
M = 1.80;  % Mass of kite (included payload=0)
A = 10; % Area of kite
l_t = 7.5; % tether length


r_0 = [0 0 1 ]'.*l_t;% initial position of kite
theta_0 = 1.2*beta;%60*pi/180; %pi/3;%beta;      % initial position of kite in spherical
dtheta_0 = 0;        % initial velocity of kite in spherical
phi_0 = 0;           % initial position of kite in spherical
dphi_0 = 0;          % initial velocity of kite in spherical
theta_max = pi/2;    % Saturation on elevation angle (hit ground)
theta_min = 0;       % Saturation on elevation angle (cant go against the wind)
phi_max = pi/2;
phi_min = -pi/2;
% Air Rudder parameters
Ear = 20;

% Tether properties
CT = 1.2;    % Normal reaction coefficient
d_t = 0.01;           % Diameter
m_tether = 0;

% Control bar properties
m_c = 0.66; % [kg]
l_c = 1.0;  % [m]

% PID controller values
x1_0 = beta;   
x3_0 = 0;%pi/8;

% Abrevated parameters
Ge = CL/(CD + (l_t*d_t*CT)/(4*A));    % Effective glide ratio
kappa_g = 0;    % Geodesic curvature of trajectory

% Physical properties
rho_a = 1.23;    % density of air kg/m^3
g = 9.81;

% Ambient wind
W = [3 0 0]';      % NED ambient wind velocity
z_0 = 10;           % reference height dynamic wind formula.
alpha = 1/7;
noise_power = 0.01;    %0.01

set(0,'defaulttextinterpreter','latex');
if plotter
    sim('Openloopkite2',T_sim)
    figure(1)
    plot3(posout.Data(:,1),posout.Data(:,2),posout.Data(:,3));
    %title('Kite 3D trajectory');
    axis([0 l_t -l_t l_t 0 l_t]);
    xlabel('$x$');
    ylabel('$y$');
    zlabel('$z$');
    
    % Save fig 2 file as bitmap 600 dpi resolution
    if savefigs
        fig2img( 600, 1 );
    end
    
    figure(2)
    subplot(3,3,1)
    plot(thetavec.Time,thetavec.Data(:,1),[thetavec.Time(1) thetavec.Time(end)],[pi/2 pi/2],'r',[thetavec.Time(1) thetavec.Time(end)],[0 0],'r',[thetavec.Time(1) thetavec.Time(end)],[x1_0 x1_0],'g');
    xlabel('$\theta$');
    subplot(3,3,2)
    plot(thetavec.Time,thetavec.Data(:,2));
    xlabel('$\dot{ \theta}$');
    subplot(3,3,3)
    plot(thetavec.Time,thetavec.Data(:,3));
    xlabel('$\ddot{ \theta }$');
    
    
    subplot(3,3,4)
    plot(phivec.Time,phivec.Data(:,1),[phivec.Time(1) phivec.Time(end)],[phi_min phi_min],'r',[phivec.Time(1) phivec.Time(end)],[phi_max phi_max],'r',[phivec.Time(1) phivec.Time(end)],[x3_0 x3_0],'g');
    xlabel('$\phi$');
    subplot(3,3,5)
    plot(phivec.Time,phivec.Data(:,2));
    xlabel('$\dot{ \phi }$');
    subplot(3,3,6)
    plot(phivec.Time,phivec.Data(:,3));
    xlabel('$\ddot{ \phi }$');
    
    subplot(3,3,7)
    plot(detXtX.Time, detXtX.Data );
    xlabel('det($\chi^T \chi$)');
    
    % Save fig 2 file as bitmap 600 dpi resolution
    if savefigs
        fig2img( 600, 1 );
    end
    
    figure(3)
    subplot(2,2,1)
    plot(xis.Time,xis.Data(:,1),[xis.Time(1) xis.Time(end)],[0.5 0.5],'r',[xis.Time(1) xis.Time(end)],[-0.5 -0.5],'r')
    xlabel('$\xi_1$');
    axis([xis.Time(1) xis.Time(end) -1 1])
    subplot(2,2,2)
    plot(xis.Time,xis.Data(:,2),[xis.Time(1) xis.Time(end)],[0.5 0.5],'r',[xis.Time(1) xis.Time(end)],[-0.5 -0.5],'r')
    xlabel('$\xi_2$');
    axis([xis.Time(1) xis.Time(end) -1 1])
    subplot(2,2,3)
    plot(xis.Time,xis.Data(:,3),[xis.Time(1) xis.Time(end)],[0.5 0.5],'r',[xis.Time(1) xis.Time(end)],[-0.5 -0.5],'r')
    xlabel('$\xi_3$');
    axis([xis.Time(1) xis.Time(end) -1 1])
    subplot(2,2,4)
    plot(xis.Time,xis.Data(:,4),[xis.Time(1) xis.Time(end)],[0.5 0.5],'r',[xis.Time(1) xis.Time(end)],[-0.5 -0.5],'r')
    xlabel('$\xi_4$');
    axis([xis.Time(1) xis.Time(end) -1 1])
   
    % Save fig 2 file as bitmap 600 dpi resolution
    if savefigs
        fig2img( 600, 1 );
    end
    
end

if plotTetherTension
    
    figure(4)
    %sim('Openloopkite2',T_sim)
    subplot(2,2,1)
    plot(FcOUT.Time, -FcOUT.Data(:,1) );
    %title('Tether Constraint Force' );
    xlabel('$F_c [N]$');
    
    subplot(2,2,4)
    plot(thetavec.Time, thetavec.Data(:,3).*l_c*m_c*0.5 );
    xlabel('$\tau_\theta$ [N]');
    
    subplot(2,2,3)
    plot(phivec.Time,phivec.Data(:,3).*l_c*m_c*0.5 );
    xlabel('$\tau_\phi$ [N]');
    
%     subplot(3,2,5)
%     plot(u1.Time, u1.Data );
%     xlabel( '$u_1(t)$');
%     
%     subplot(3,2,6)
%     plot(u2.Time, u2.Data );
%     xlabel( '$u_2(t)$' );
    
    % Save fig 2 file as bitmap 600 dpi resolution
    if savefigs
        fig2img( 600, 1 );
    end
end
