clear;
N = 30; % Number of elements per input in controllability analysis
closetest = 0;


h = waitbar(0,'Initializing sym objects...');

syms x1 x2 x3 x4 u1 u2; %w
%syms l w m g rho_a CD CL Cnorm Ak dt;
syms f Fx1 Fx3 G a b c d H I FD FL psi1 psi2;

assume(x1 > 0 & x1 < pi/2 );
assume(x2,'real');
assume(x3 > -pi/2 & x3 < pi/2 );
assume(x4,'real');
assume(u1 > -pi & u1 < pi );
assume(u2,'real');

assume(f, 'real');
assume(Fx1, 'real');
assume(Fx3, 'real');
assume(G, 'real');
assume(a, 'real');
assume(b, 'real');
assume(c, 'real');
assume(d, 'real');
assume(H, 'real');
assume(I, 'real');
assume(FD, 'real');
assume(FL, 'real');
assume(psi1, 'real');
assume(psi2, 'real');

waitbar(1/10,h,'Simplifying first group...');

l = 7.5;% assume(l > 0 );
% assume(w > 0 );
m = 1.80;% mass of kite. Mass of ctrl-bar: 0.66;
g = 9.81;% assume(g > 0 );
rho_a = 1.23;% assume(rho_a > 0);
CL = 1.5;% assume(CD > 0 );
Ee = 3;%effective glide ratio
CD = CL/Ee;% assume(CL > 0 );
Cnorm = 1.2;% assume( Cnorm > 0 );
Ak = 10;% assume( Ak > 0 );
dt = 0.01;% assume( dt > 0 );
bias = eps;
w = 3;      % ambient wind speed [m/s] in x-direction

er = [sin(x1)*cos(x3);sin(x1)*sin(x3);cos(x1)];
etheta = [cos(x1)*cos(x3);cos(x1)*sin(x3);-sin(x1)];
ephi = [-sin(x3);cos(x3);0];

W = [w 0 0]';
p = l.*er;
dp = l.*etheta.*x2 + l*sin(x1).*ephi.*x4;
We = W - dp;
Wep = We - dot(er,We).*er;
simplify(Wep,'IgnoreAnalyticConstraints',true,'Steps',10);
el = We/norm(We);
et = cross(We,er)/norm(cross(We,er));
et = simplify(et,'IgnoreAnalyticConstraints',true,'Steps',10);
% epsi = cross(et,el);
% simplify(epsi,'IgnoreAnalyticConstraints',true,'Steps',10);
epsi = cross(cross(We,er),We)/(norm(We)*norm(cross(We,er)));
simplify(epsi,'IgnoreAnalyticConstraints',true,'Steps',10);

waitbar(2/10,h,'Simplifying second group...');

FD = 0.5*rho_a*CD*Ak*dot(We,We);
FD = simplify(FD, 'Steps', 10, 'IgnoreAnalyticConstraints',true);
FL = 0.5*rho_a*CL*Ak*dot(We,We);
FL = simplify(FL, 'Steps', 10, 'IgnoreAnalyticConstraints',true);
Fdt = eval(1/8*rho_a*Cnorm*dt*l*norm(Wep));
Fdt = simplify(Fdt, 'Steps', 10);%, 'IgnoreAnalyticConstraints',true)

waitbar(3/10,h,'Simplifying third group...');

psi1 = dot(We,etheta)/norm(We);
psi1 = simplify(psi1, 'Steps', 10, 'IgnoreAnalyticConstraints',true);
psi2 = dot(ephi,We)/norm(We);
psi2 = simplify(psi2, 'Steps', 10, 'IgnoreAnalyticConstraints',true);
psi3 = dot(cross(cross(We,er),We),etheta)/dot(We,We);
psi3 = simplify(psi3, 'Steps', 10, 'IgnoreAnalyticConstraints',true);

waitbar(4/10,h,'Simplifying 4th group...');

psi4 = dot(We,ephi)/norm(We);
psi4 = simplify(psi4, 'Steps', 10,'IgnoreAnalyticConstraints',true);
psi5 = dot(etheta,We)/norm(We);
psi5 = simplify(psi5, 'Steps', 10,'IgnoreAnalyticConstraints',true);
psi6 = dot(cross(cross(We,er),We),ephi)/dot(We,We);
psi6 = simplify(psi6, 'Steps', 10,'IgnoreAnalyticConstraints',true);

waitbar(5/10,h,'Simplifying 5th group...');

psi7 = dot(We,etheta);
psi7 = simplify(psi7, 'Steps', 10,'IgnoreAnalyticConstraints',true);
psi8 = dot(We,ephi);
psi8 = simplify(psi8, 'Steps', 10,'IgnoreAnalyticConstraints',true);
% psi9 = dot(We,er)/norm(We);
% psi9 = simplify(psi9, 'Steps', 10,'IgnoreAnalyticConstraints',true);
% psi10 = norm(cross(We,er))/norm(We);
% psi10 = simplify(psi10, 'Steps', 10,'IgnoreAnalyticConstraints',true);
%
waitbar(6/10,h,'Simplifying 6th group...');

Fx1 = (FD*cos(u2)*cos(u1) - FL*sin(u2)*cos(u1))*psi1...
    + (FD*cos(u2)*sin(u1) - FL*sin(u2)*sin(u1))*psi2...
    + (FD*sin(u2) + FL*cos(u2))*psi3...
    + Fdt*psi7;
Fx1 = simplify(Fx1,'Steps',10);
Fx3 = (FD*cos(u2)*cos(u1) - FL*sin(u2)*cos(u1))*psi4...
    + (FD*cos(u2)*sin(u1) - FL*sin(u2)*sin(u1))*psi5...
    + (FD*sin(u2) + FL*cos(u2))*psi6...
    + Fdt*psi8;
Fx3 = simplify(Fx3,'Steps',10);


f = [x2;Fx1/(l*m) + sin(x1)*g/l + sin(x1)*cos(x1)*x4^2 + u2; x4;Fx3/(l*m) - 2*cot(x1)*x2*x4];

waitbar(7/10,h,'Computing Jacobian matrices A and B...');

A = jacobian( f, [x1 x2 x3 x4] );
B = jacobian( f, [u1 u2] );

waitbar(8/10,h,'Simplifying Jacobian...');

for i = 1:4
    for j = [2 4]
        A(i,j) = simplify(A(i,j), 'Steps', 2, 'IgnoreAnalyticConstraints',true);
    end
end



% fid = fopen( 'Ageneral2.dat','w');
% for i = 1:4
%     for j = 1:4
%         fprintf( fid, '%s\n', char(A(i,j)) );
%     end
% end
% fclose(fid);

% Controllability analysis at W = 5 m/s and u2 = 0
waitbar(9/10,h,'Substituting w and u2 as constants in jacobians...');
A = subs(A,u2,0);
B = subs(B,u2,0);
close(h)

CTB = zeros(N,N);

x_1 = linspace(0,pi/2,N);
x_2 = pi;
x_3 = linspace(-pi/2,pi/2,N);
x_4 = pi;
u_1 = 0;
%u_2 = 0;

h = waitbar(0,'Progress of controllability check...');
for i = 1:N
    %disp([num2str(i),'/',num2str(N),' outer iterations in 2D (N^2)']);
    if (i==1)
        time0 = tic;
    end
    for j = 1:N
        dumA = eval(subs(A,{x1,x2,x3,x4,u1},{x_1(i),x_2,x_3(j),x_4,u_1}));
        dumB = eval(subs(B,{x1,x2,x3,x4,u1},{x_1(i),x_2,x_3(j),x_4,u_1}));
        
        dumNan = isnan(dumA);
        if (max(max(dumNan)))
            CTB(i,j) = -1;
        else
            CTB(i,j) = length(dumA) - rank(ctrb(dumA,dumB));
        end
    end
    
    if(i>=2)
        tfin = toc(time0);
        ttotal = round((N/i)*tfin - tfin);
        totalm = floor(ttotal/60);
        totals = ttotal - totalm*60;
        waitbar(i/N,h,['Progress of controllability check... [ETA: ', num2str(totalm), 'm ',num2str(totals),'s]']);
    end
    
    
end
close(h);

% Graphical representation of controllability analysis results
x_12 = linspace(0,pi/2,N+1);
x_32 = linspace(-pi/2,pi/2,N+1);

zs = repmat(cos(x_12),N+1,1)';
for i = 1:(N+1)
    for j = 1:(N+1)
        xs(i,j) = sin(x_12(i))*cos(x_32(j));
        ys(i,j) = sin(x_12(i))*sin(x_32(j));
    end
end
set(0,'defaulttextinterpreter','none');
fig1 = figure(1)
surf(xs,ys,zs,CTB), axis equal
colormap([1 0 0;0 1 0 ])
xl1 = xlabel('$x$');set(xl1,'interpreter', 'latex');
yl1 = ylabel('$y$');set(yl1,'interpreter', 'latex');
zl1 = zlabel('$z$');set(zl1,'interpreter', 'latex');

%t1 = title('Controllability check for $\forall (x_1,x_3) $ at $x_2=x_4=\pi, u_1=u_2=0$ and $w = 5 [m/s]$'); set(t1,'interpreter', 'latex');
colorbar('YTickLabel',{'','Uncontrollable','','','','','','','','Fully controllable'})
fig2img( 600, 1 );

% Running controllability test in the close range near x1 = 0 and x1 = pi/2

if closetest
    CTBc1 = zeros(N,N);
    CTBc2 = zeros(N,N);
    
    x_13 = linspace(x_1(1),x_1(2),N);
    x_14 = linspace(x_1(end),x_1(end-1),N);
    
    for i = 1:N
        for j = 1:N
            dumA1 = eval(subs(A,{x1,x2,x3,x4,u1},{x_13(i),x_2,x_3(j),x_4,u_1}));
            dumB1 = eval(subs(B,{x1,x2,x3,x4,u1},{x_13(i),x_2,x_3(j),x_4,u_1}));
            
            dumA2 = eval(subs(A,{x1,x2,x3,x4,u1},{x_14(i),x_2,x_3(j),x_4,u_1}));
            dumB2 = eval(subs(B,{x1,x2,x3,x4,u1},{x_14(i),x_2,x_3(j),x_4,u_1}));
            
            dumNanA1 = isnan(dumA1);
            dumNanB1 = isnan(dumB1);
            
            dumNanA2 = isnan(dumA2);
            dumNanB2 = isnan(dumB2);
            
            if max(max(dumNanA1))
                CTBc1(i,j) = -1;
            else
                CTBc(i,j) = length(dumA1) - rank(ctrb(dumA1,dumB1));
            end
            
            if max(max(dumNanA2))
                CTBc2(i,j) = -1;
            else
                CTBc2(i,j) = length(dumA2) - rank(ctrb(dumA2,dumB2));
            end
        end
    end
    
end

